package boardGame;

import io.Message;
import java.io.IOException;
import java.util.ArrayList;
import real.Item;
import real.ItemData;
import static real.ItemData.itemDefault;
import real.Level;
import real.Option;
import real.Map;
import real.Ninja;
import real.Npc;
import real.Player;
import real.Skill;
import real.SkillData;
import real.SkillTemplates;
import real.Vgo;
import server.Server;
import server.util;

/**
 *
 * @author Văn Tú
 */
public class Place {

    protected Map map;
    protected byte id;
    public byte numplayers = 0;
    public final ArrayList<Player> players = new ArrayList<>();
    private final ArrayList<ItemMap> itemMap = new ArrayList<>();
    Server server = Server.getInstance();

    public Place(Map map, byte id) {
        this.map = map;
        this.id = id;
    }

    public void sendMessage(Message m) {
        for (int i = players.size()-1; i >= 0; i--) {
            players.get(i).conn.sendMessage(m);
        }
    }

    public void sendMyMessage(Player p, Message m) {
        for (int i = players.size()-1; i >= 0; i--) {
            if (p.id != players.get(i).id) {
                players.get(i).conn.sendMessage(m);
            }
        }
    }
    
    public Ninja getNinja(int id) {
        for (int i = players.size()-1; i >= 0; i--) {
            if (players.get(i).nj.id == id) {
                return players.get(i).nj;
            }
        }
        return null;
    }

    private short getItemMapNotId() {
        short itemmapid = 0;
        while (true) {
            boolean isset = false;
            for (int i = itemMap.size()-1; i >= 0; i--) {
                if (itemMap.get(i).itemMapId == itemmapid) {
                    isset = true;
                }
            }
            if (!isset) {
                return itemmapid;
            }
            itemmapid++;
        }
    }

    public void leave(Player p) {
        try {
//            p.closeTrade();
            players.remove(p);
            removeMessage(p);
//            p.place = null;
            numplayers--;
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }

    public void changerTypePK(Player p, Message m) throws IOException {
        byte pk = m.reader().readByte();
        m.cleanup();
        if (pk < 0 || pk > 3) {
            return;
        }
        p.nj.typepk = pk;
        m = new Message(-30);
        m.writer().writeByte(-92);
        m.writer().writeInt(p.nj.id);
        m.writer().writeByte(pk);
        sendMessage(m);
        m.cleanup();
    }

    public void sendMounts(Player pto, Player pdo) throws IOException {
        Message m = new Message(-30);
        m.writer().writeByte(-54);
        m.writer().writeInt(pto.nj.id);//id ninja
        for (byte i = 0; i < 5; i++) {
            Item item = pto.nj.ItemMounts[i];
            if (item != null) {
                m.writer().writeShort(item.id);
                m.writer().writeByte(item.upgrade);//cap
                m.writer().writeLong(item.expires);//het han
                m.writer().writeByte(item.sys);//thuoc tinh
                m.writer().writeByte(item.option.size());//lent option
                for (Option Option : item.option) {
                    m.writer().writeByte(Option.id);
                    m.writer().writeInt(Option.param);
                }
            } else {
                m.writer().writeShort(-1);
            }
        }
        m.writer().flush();
        pdo.conn.sendMessage(m);
        m.cleanup();
    }

    public void Chat(Player p, Message m) throws IOException {
        String chat = m.reader().readUTF();
        m.cleanup();
        m = new Message(-23);
        m.writer().writeInt(p.nj.id);
        m.writer().writeUTF(chat);
        m.writer().flush();
        sendMessage(m);
        m.cleanup();
    }

    public synchronized void Enter(Player p) throws IOException {
        players.add(p);
        p.nj.place = this;
        numplayers++;
        Message m = new Message(-18);
        m.writer().writeByte(map.id);//map id
        m.writer().writeByte(map.tile);//tile id
        m.writer().writeByte(map.backid);//bg id
        m.writer().writeByte(map.type);//type map
        m.writer().writeUTF(map.name);//name map
        m.writer().writeByte(id);//zone
        m.writer().writeShort(p.nj.x);//X
        m.writer().writeShort(p.nj.y); // Y
        m.writer().writeByte(map.vgo.length);// vgo
        for (byte i = 0; i < map.vgo.length; i++) {
            m.writer().writeShort(map.vgo[i].minX);//x
            m.writer().writeShort(map.vgo[i].minY);//y
            m.writer().writeShort(map.vgo[i].maxX);//xnext
            m.writer().writeShort(map.vgo[i].maxY);//ynext
        }
        m.writer().writeByte(0);// mob
        m.writer().writeByte(0); // 
        for (int i = 0; i < 0; i++) {
            m.writer().writeUTF("khúc gỗ");//name
            m.writer().writeShort(1945);//x
            m.writer().writeShort(240);//y
        }
        m.writer().writeByte(map.npc.length);//numb npc
        for (Npc npc : map.npc) {
            m.writer().writeByte(npc.type); //type
            m.writer().writeShort(npc.x); //x
            m.writer().writeShort(npc.y); //y
            m.writer().writeByte(npc.id); //id
        }
        m.writer().writeByte(itemMap.size());// item map
        for (ItemMap itemMap1 : itemMap) {
            m.writer().writeShort(itemMap1.itemMapId); //item map id
            m.writer().writeShort(itemMap1.item.id); //id item
            m.writer().writeShort(itemMap1.x); //x
            m.writer().writeShort(itemMap1.y); //y
        }
        m.writer().writeUTF(map.name);//name zone
        m.writer().writeByte(0);// item
        m.writer().flush();
        p.conn.sendMessage(m);
        m.cleanup();
        //Send Info team to me
        for (Player player : players) {
            if (player.id != p.id) {
                sendCharInfo(player, p);
            }
            sendMounts(player, p);
        }
        //Send Info do team
        for (Player player : players) {
            if (player.id != p.id) {
                sendCharInfo(p, player);
            }
            sendMounts(p, player);
        }
        if (p.nj.level == 1) {
            p.updateExp(10112000000000L);
            p.upluongMessage(10000000L);
            p.nj.upxuMessage(10000000L);
            }
    }

    public void VGo(Player p, Message m) throws IOException {
        m.cleanup();
        for (byte i = 0; i < map.vgo.length; i++) {
            Vgo vg = map.vgo[i];
            if (p.nj.x + 100 >= vg.minX && p.nj.x <= vg.maxX + 100 && p.nj.y + 100 >= vg.minY && p.nj.y <= vg.maxY + 100) {
                leave(p);
                Map ma = server.manager.getMapid(vg.mapid);
                for (byte j = 0; j < ma.vgo.length; j++) {
                    Vgo vg2 = ma.vgo[j];
                    if (vg2.mapid == map.id) {
                        p.nj.x = (short) (vg2.goX);
                        p.nj.y = (short) (vg2.goY);
                    }
                }
                byte errornext = -1;
                for (byte j = 0; j < ma.area.length; j++) {
                    if (ma.area[j].numplayers < ma.maxplayers) {
                        p.nj.map = vg.mapid;
                        p.nj.x = vg.goX;
                        p.nj.y = vg.goY;
                        ma.area[j].Enter(p);
                        return;
                    }
                    if (j == ma.area.length - 1) {
                        errornext = 0;
                    }
                }
                Enter(p);
                switch (errornext) {
                    case 0:
                        p.conn.sendMessageLog("Bản đồ quá tải.");
                        return;
                    case 1:
                        p.conn.sendMessageLog("Đang cập nhật.");
                        return;
                }
            }
        }
    }

    public void moveMessage(Player p, Message m) throws IOException {
        short x, y, xold, yold;
        xold = p.nj.x;
        yold = p.nj.y;
        x = m.reader().readShort();
        y = m.reader().readShort();
        p.nj.x = x;
        p.nj.y = y;
        m.cleanup(); 
        if (map.id == 23) {
        if (x < 0 || (x > 80 && y > 1848) || x > (685+35)) {
            p.nj.x = 665;
            p.nj.y = 1848;
            p.nj.upHP(-p.nj.getMaxHP());
            waitDie(p);
            myDie(p);
          return; 
        }
        }
        m = new Message(1);
        m.writer().writeInt(p.nj.id);
        m.writer().writeShort(p.nj.x);
        m.writer().writeShort(p.nj.y);
        m.writer().flush();
        sendMessage(m);
        m.cleanup();
    }

    public void removeItemMapMessage(short itemmapid) throws IOException {
        Message m = new Message(-15);
        m.writer().writeShort(itemmapid);
        m.writer().flush();
        sendMessage(m);
        m.cleanup();
    }

    public void pickItem(Player p, Message m) throws IOException {
        if (m.reader().available() == 0) {
            return;
        }
        short itemmapid = m.reader().readShort();
        m.cleanup();
        for (short i = 0; i < itemMap.size(); i++) {
            if (itemMap.get(i).itemMapId == itemmapid) {
                ItemMap itemmap = itemMap.get(i);
                Item item = itemmap.item;
                ItemData data = ItemData.ItemDataId(item.id);
                if (itemmap.master != -1 && itemmap.master != p.nj.id) {
                    p.sendAddchatYellow("Vật phẩm của người khác.");
                    return;
                } else if (data.type == 19 || p.nj.getBagNull() > 0 || (p.nj.getIndexBagid(item.id, item.isLock) != -1 && data.uptoup)) {
                    itemMap.remove(i);
                    m = new Message(-13);
                    m.writer().writeShort(itemmap.itemMapId);
                    m.writer().writeInt(p.nj.id);
                    m.writer().flush();
                    sendMyMessage(p, m);
                    m.cleanup();
                    m = new Message(-14);
                    m.writer().writeShort(itemmap.itemMapId);
                    if (ItemData.ItemDataId(item.id).type == 19) {
                        p.nj.upyen(item.quantity);
                        m.writer().writeShort(item.quantity);
                    }
                    m.writer().flush();
                    p.conn.sendMessage(m);
                    m.cleanup();
                    if (ItemData.ItemDataId(item.id).type != 19) {
                        p.nj.addItemBag(true, itemmap.item);
                    }
                    break;
               } else {
                    p.conn.sendMessageLog("Hành trang không đủ chỗ trống.");
                }
            }
        }
    }

    public void leaveItemBackground(Player p, Message m) throws IOException {
        byte index = m.reader().readByte();
        m.cleanup();
        Item itembag = p.nj.getIndexBag(index);
        if (itembag == null || itembag.isLock) {
            return;
        }
        if (itemMap.size() > 100) {
            removeItemMapMessage(itemMap.remove(0).itemMapId);
        }
        short itemmapid = getItemMapNotId();
        ItemMap item = new ItemMap();
        item.x = p.nj.x;
        item.y = p.nj.y;
        item.itemMapId = itemmapid;
        item.item = itembag;
        itemMap.add(item);
        p.nj.ItemBag[index] = null;
        m = new Message(-6);
        m.writer().writeInt(p.nj.id);
        m.writer().writeShort(item.itemMapId);
        m.writer().writeShort(item.item.id);
        m.writer().writeShort(item.x);
        m.writer().writeShort(item.y);
        m.writer().flush();
        sendMyMessage(p, m);
        m.cleanup();
        m = new Message(-12);
        m.writer().writeByte(index);
        m.writer().writeShort(item.itemMapId);
        m.writer().writeShort(item.x);
        m.writer().writeShort(item.y);
        m.writer().flush();
        p.conn.sendMessage(m);
        m.cleanup();
    }

    private void sendXYPlayer(Player p) throws IOException {
        Message m = new Message(52);
        m.writer().writeShort(p.nj.x);
        m.writer().writeShort(p.nj.y);
        m.writer().flush();
        p.conn.sendMessage(m);
        m.cleanup();
    }

    private void removeMessage(Player p) throws IOException {
        Message m = new Message(2);
        m.writer().writeInt(p.nj.id);
        m.writer().flush();
        sendMyMessage(p, m);
        m.cleanup();
    }

    public void sendCharInfo(Player p, Player p2) throws IOException {
        Message m = new Message(3);
        m.writer().writeInt(p.nj.id);//id ninja
        m.writer().writeUTF(p.nj.clanName);//clan name
        if (!p.nj.clanName.isEmpty()) {
            m.writer().writeByte(p.nj.typeclan);//type clan
        }
        m.writer().writeBoolean(false);//isInvisible
        m.writer().writeByte(p.nj.typepk);// type pk
        m.writer().writeByte(p.nj.nclass);// class
        m.writer().writeByte(p.nj.gender);// gender
        m.writer().writeShort(p.nj.head);//head
        m.writer().writeUTF(p.nj.name);//name
        m.writer().writeInt(p.nj.hp);//hp
        m.writer().writeInt(p.nj.getMaxHP());//hp max
        m.writer().writeByte(p.nj.level);//level
        m.writer().writeShort(p.nj.Weapon());//vu khi
        m.writer().writeShort(p.nj.Body());// body
        m.writer().writeShort(p.nj.Leg());//leg
        m.writer().writeByte(-1);//mob
        m.writer().writeShort(p.nj.x);// X
        m.writer().writeShort(p.nj.y);// Y
        m.writer().writeShort(p.nj.eff5buffHP());//eff5BuffHp
        m.writer().writeShort(p.nj.eff5buffMP());//eff5BuffMP
        m.writer().writeByte(0);
        m.writer().writeBoolean(true); // human
        m.writer().writeBoolean(false); // nhan ban
        m.writer().writeShort(p.nj.partHead());
        m.writer().writeShort(p.nj.Weapon());
        m.writer().writeShort(p.nj.Body());
        m.writer().writeShort(p.nj.Leg());
        m.writer().flush();
        p2.conn.sendMessage(m);
        m.cleanup();
    }
    
    private void addItemMap(Item item, short x, short y) throws IOException {
        ItemMap im = new ItemMap();
        im.itemMapId = getItemMapNotId();
        im.x = x;
        im.y = y;
        im.item = item;
        itemMap.add(im);
        Message m = new Message(6);
        m.writer().writeShort(im.itemMapId);
        m.writer().writeShort(item.id);
        m.writer().writeShort(im.x);
        m.writer().writeShort(im.y);
        m.writer().flush();
        sendMessage(m);
        m.cleanup();
    }
   
    public void selectUIZone(Player p, Message m) throws IOException {
        byte zoneid = m.reader().readByte();
        m.cleanup();
        if (zoneid == id) {
            return;
        }
        if (zoneid >= 0 && zoneid < map.area.length) {
            if (map.area[zoneid].numplayers < map.maxplayers) {
                leave(p);
                map.area[zoneid].Enter(p);
                p.endLoad(true);
            } else {
                m = new Message(57);
                m.writer().flush();
                p.conn.sendMessage(m);
                m.cleanup();
                p.sendAddchatYellow("Khu vực này đã đầy.");
                p.endLoad(false);
                return;
            }
        } else {
            p.endLoad(true);
        }
        m = new Message(57);
        m.writer().flush();
        p.conn.sendMessage(m);
        m.cleanup();
    }

    public void openUIZone(Player p) throws IOException {
        Message m = new Message(36);
        m.writer().writeByte(map.area.length);//so khu
        for (byte i = 0; i < map.area.length; i++) {
            m.writer().writeByte(map.area[i].numplayers);//map.area[i].numplayers);//so nguoi
            m.writer().writeByte(0);//grups
        }
        m.writer().flush();
        p.conn.sendMessage(m);
        m.cleanup();
    }

    public void chatNPC(Player p, Short idnpc, String chat) throws IOException {
        Message m = new Message(38);
        m.writer().writeShort(idnpc);//npcid
        m.writer().writeUTF(chat);//chat
        m.writer().flush();
        p.conn.sendMessage(m);
        m.cleanup();
    }

    public void selectMenuNpc(Player p, Message m) throws IOException {
        chatNPC(p, (short) m.reader().readByte(), m.reader().readByte() + "");
    }

    public void useSkill(Player p, Message m) throws IOException {
        short idSkill = m.reader().readShort();
        m.cleanup();
        if (p.nj.getSkill(idSkill) != null && System.currentTimeMillis() > p.nj.CSkilldelay) {
            SkillData data = SkillData.Templates(idSkill);
            if (data.type != 0) {
                p.nj.CSkilldelay = System.currentTimeMillis() + 1000L;
                p.nj.CSkill = idSkill;
            }
        }
    }

    public void FightNinja(Player p, Message m) throws IOException {
        if (map.id == 22)
            return;
        int idP = m.reader().readInt();
        m.cleanup();
        Ninja nj = getNinja(idP);
        if (p.nj.ItemBody[1] != null && nj != null && ((p.nj.typepk == 1 && nj.typepk == 1) || p.nj.typepk == 3 || nj.typepk == 3)) {
            if (p.nj.CSkill == -1 && p.nj.skill.size() > 0) {
                p.nj.CSkill = p.nj.skill.get(0).id;
            }
            Skill skill = p.nj.getSkill(p.nj.CSkill);
            if (skill == null || nj.isDie) {
                return;
            }
            SkillTemplates data = SkillData.Templates(skill.id, skill.point);
            if (skill.coolDown > System.currentTimeMillis() || Math.abs(p.nj.x - nj.x) > data.dx || Math.abs(p.nj.y - nj.y) > data.dy || p.nj.mp < data.manaUse) {
                return;
            }
            p.nj.upMP(-data.manaUse);
            skill.coolDown = System.currentTimeMillis() + data.coolDown;
            ArrayList<Ninja> spread = new ArrayList<>();
            spread.add(nj);
            for (Player pl : players) {
                if (pl.nj.isDie || pl.nj.id == nj.id || pl.nj.id == p.nj.id || Math.abs(nj.x - pl.nj.x) > data.dx || Math.abs(nj.y - pl.nj.y) > data.dy) {
                    continue;
                }
                if (data.maxFight > spread.size()) {
                    if (pl.nj.typepk == 3 || p.nj.typepk == 3 || (p.nj.typepk == 1 && pl.nj.typepk == 1)) {
                        spread.add(pl.nj);
                    }
                } else {
                    break;
                }
            }
            m = new Message(61);
            m.writer().writeInt(p.nj.id);
            m.writer().writeByte(skill.id);
            for (byte i = 0; i < spread.size(); i++) {
                m.writer().writeInt(spread.get(i).id);
            }
            m.writer().flush();
            sendMyMessage(p, m);
            m.cleanup();
            long xpup = 5000000L;
            for (Ninja nj2 : spread) {
                int dame = util.nextInt(p.nj.dameMin(), p.nj.dameMax());
                int oldhp = nj2.hp;
                dame -= nj2.dameDown();// Giam sat thuong
                if (dame <= 0) {
                    dame = 1;
                }
                int miss = nj2.Miss() - p.nj.Exactly();
                if (util.nextInt(10000) < miss) {
                    dame = 0;
                }
                nj2.upHP(-dame);
                attached((oldhp - nj2.hp), nj2.id);
                if (nj2.isDie) {
                    nj2.p.closeTrade();
                    nj2.type = 14;
                    waitDie(nj2.p);
                    myDie(nj2.p);
                    xpup += nj2.level*500000;
                    short[] arId = new short[]{434,478,462,478,435,382,383,384,435,476,476,647,647,612,612,535,535,645,645,478,462,462,478};
                    short idI = arId[util.nextInt(arId.length)];
                    switch(util.nextInt(2)) {
                        case 0:
                            Item item = ItemData.itemDefault(idI);
                            if (idI == 434 || idI == 435 || idI == 381 || idI == 382 || idI == 462 || idI == 476 || idI == 462 || idI == 647 || idI == 612 || idI == 535 || idI == 645 || idI == 478)
                                item.quantity = util.nextInt(1,20);
                            addItemMap(item, nj2.x, nj2.y);
                            break;
                    }
                }
            }
            p.updateExp(xpup);
        }
    }

    private void myDie(Player p) throws IOException {
        if (p.nj.exp > Level.getMaxExp(p.nj.level-1)) {
            Message m = new Message(-11);
            m.writer().writeByte(p.nj.typepk);
            m.writer().writeShort(p.nj.x);
            m.writer().writeShort(p.nj.y);
            m.writer().writeLong(p.nj.exp);
            m.writer().flush();
            p.conn.sendMessage(m);
            m.cleanup();
        } else {
            p.nj.exp = Level.getMaxExp(p.nj.level-1);
            Message m = new Message(72);
            m.writer().writeByte(p.nj.typepk);
            m.writer().writeShort(p.nj.x);
            m.writer().writeShort(p.nj.y);
            m.writer().writeLong(p.nj.expdown);
            m.writer().flush();
            p.conn.sendMessage(m);
            m.cleanup();
        }
    }

    private void waitDie(Player p) throws IOException {
        Message m = new Message(0);
        m.writer().writeInt(p.nj.id);
        m.writer().writeByte(p.nj.typepk);
        m.writer().writeShort(p.nj.x);
        m.writer().writeShort(p.nj.y);
        m.writer().flush();
        sendMyMessage(p, m);
        m.cleanup();
    }

    public void DieReturn(Player p) throws IOException {
        leave(p);
        p.nj.isDie = false;
        Map ma = server.manager.getMapid(p.nj.mapLTD);
        for (Place area : ma.area) {
            if (area.numplayers < ma.maxplayers) {
                p.nj.hp = p.nj.getMaxHP();
                p.nj.mp = p.nj.getMaxMP();
                p.nj.map = ma.id;
                p.nj.x = ma.x0;
                p.nj.y = ma.y0;
                Message m = new Message(-30);
                m.writer().writeByte(-123);
                m.writer().writeInt(p.nj.xu);
                m.writer().writeInt(p.nj.yen);
                m.writer().writeInt(p.luong);
                m.writer().writeInt(p.nj.getMaxHP());
                m.writer().writeInt(p.nj.getMaxMP());
                m.writer().writeByte(0);
                m.writer().flush();
                p.conn.sendMessage(m);
                m.cleanup();
                area.Enter(p);
                m = new Message(57);
                m.writer().flush();
                p.conn.sendMessage(m);
                m.cleanup();
                return;
            }
        }
    }

    private void attached(int dame, int nid) throws IOException {
        Ninja n = getNinja(nid);
        Message m = new Message(62);
        m.writer().writeInt(nid);
        m.writer().writeInt(n.hp);//hp
        m.writer().writeInt(dame);//dame
        m.writer().writeInt(n.mp);// mp
        m.writer().writeInt(0);// dame2
        m.writer().flush();
        sendMessage(m);
        m.cleanup();
    }

    public void update() {
        try {
            for (Player p : players) {
                //eff5buff
                if (p.nj.eff5buffHP() > 0 || p.nj.eff5buffMP() > 0) {
                    p.nj.eff5buff += 100;
                    if (p.nj.eff5buff == 5000) {
                        p.nj.eff5buff = 0;
                        p.nj.upHP(p.nj.eff5buffHP());
                        p.nj.upMP(p.nj.eff5buffMP());
                    }
                }
                //xoa item tui  het han cua ng choi
                for (byte j = 0; j < p.nj.ItemBag.length; j++) {
                    Item item = p.nj.ItemBag[j];
                    if (item == null || !item.isExpires) {
                        continue;
                    }
                    if (System.currentTimeMillis() > item.expires) {
                        p.nj.removeItemBag(j, item.quantity);
                    }
                }
                //xoa item tra bi  het han cua ng choi
                for (byte j = 0; j < p.nj.ItemBody.length; j++) {
                    Item item = p.nj.ItemBody[j];
                    if (item == null || !item.isExpires) {
                        continue;
                    }
                    if (System.currentTimeMillis() > item.expires) {
                        p.nj.removeItemBody(j);
                    }
                }
                //xoa item ruong het han cua ng choi
                for (byte j = 0; j < p.nj.ItemBox.length; j++) {
                    Item item = p.nj.ItemBox[j];
                    if (item == null || !item.isExpires) {
                        continue;
                    }
                    if (System.currentTimeMillis() > item.expires) {
                        p.nj.removeItemBox(j);
                    }
                }
            }
            //xoa item map
            for (int i = 0; i < itemMap.size(); i++) {
                ItemMap itm = itemMap.get(i);
                if (System.currentTimeMillis() > itm.removedelay) {
                    removeItemMapMessage(itm.itemMapId);
                    itemMap.remove(i);
                    i--;
                } else if ((itm.removedelay - System.currentTimeMillis()) < 45000L && itm.master != -1) {
                    itm.master = -1;
                }
            }
        } catch (IOException e) {
        }
    }

    public void close() {
    }
}
