package server;

import io.Session;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.BindException;
import java.net.ServerSocket;
import java.net.Socket;
import real.Map;
import real.PlayerManager;
import real.RealController;

/**
 *
 * @author Văn Tú
 */
public class Server {

    private static Server instance = null;
    private ServerSocket listenSocket = null;
    public static boolean start = false;

    public Manager manager;
    public MenuController menu;
    public ServerController controllerManager;
    public SessionHandler serverMessageHandler;

    public ByteArrayOutputStream[] cache = new ByteArrayOutputStream[4];
    public Map[] map;

    private void init() {
        manager = new Manager();
        menu = new MenuController();
        controllerManager = new RealController();
        serverMessageHandler = new SessionHandler();
        cache[1] = GameScr.loadFile("res/cache/data");
        cache[1] = GameScr.loadFile("res/cache/map");
        cache[2] = GameScr.loadFile("res/cache/skill");
        cache[3] = GameScr.loadFile("res/cache/item");
    }

    public static Server getInstance() {
        if (instance == null) {
            instance = new Server();
            instance.init();
        }
        return instance;
    }

    public static void main(String[] args) {
        start = true;
        getInstance().run();
    }

    public void run() {
        Runtime.getRuntime().addShutdownHook(new Thread(new Runnable() {
            @Override
            public void run() {
                System.out.println("Shutdown Server!");
                stop();
            }
        }));
        listenSocket = null;
        try {
            listenSocket = new ServerSocket(manager.post);
            System.out.println("Listen " + manager.post);
            while (start) {
                Socket clientSocket = listenSocket.accept();
                Session conn = new Session(clientSocket, serverMessageHandler);
                PlayerManager.getInstance().put(conn);
                conn.start();
                System.out.println("Accept socket size :" + PlayerManager.getInstance().conns_size());
            }
        } catch (BindException bindEx) {
            System.exit(0);
        } catch (IOException genEx) {
            genEx.printStackTrace();
        }
        try {
            if (listenSocket != null) {
                listenSocket.close();
            }
            System.out.println("Close server socket");
        } catch (Exception ioEx) {
        }
    }

    public void stop() {
        if (start) {
            //Kick all Player game
            PlayerManager.getInstance().Clear();
            start = false;
            try {
                listenSocket.close();
            } catch (IOException ex) {
                ex.printStackTrace();
            }
            manager.close();
            manager = null;
            PlayerManager.getInstance().close();
            menu = null;
            controllerManager = null;
            serverMessageHandler = null;
            SQLManager.close();
            System.gc();
        }
    }
}
